package cat.hack3.codingtests.marsrover.ui;

import cat.hack3.codingtests.marsrover.Coordinates;
import cat.hack3.codingtests.marsrover.Direction;
import cat.hack3.codingtests.marsrover.MarsMap;
import cat.hack3.codingtests.marsrover.MarsRover;

import java.util.InputMismatchException;
import java.util.Scanner;

import static cat.hack3.codingtests.marsrover.ui.UICommons.isNotExitSignal;
import static cat.hack3.codingtests.marsrover.ui.UICommons.output;

public class ClientCommandInterface {

    private static MarsRover rover;

    public static void main(String[] args) {
        output(PresentationMessage.INTRO);

        try (var reader = new Scanner(System.in)) {
            //autoInitialization();
            initialization(reader);
            commandLoop(reader);
        }
    }

    private static void autoInitialization() { //to test more quickly
        deployRover(10, 10, 2, 3, Direction.SOUTH);
    }

    private static void initialization(Scanner reader) {
        output("Please, introduce the latitude  map size:");
        int mapHeight = takeIntegerInput(reader);
        output("Please, introduce the longitude map size:");
        int mapWidth = takeIntegerInput(reader);

        output("Great! Now please introduce the initial location of the Rover.");
        output("Latitude:");
        int latitudeStartingPoint = takeIntegerInput(reader);
        output("Longitude:");
        int longitudeStartingPoint = takeIntegerInput(reader);

        Direction startingDirection = DirectionRetriever.retrieveDirection(reader);

        deployRover(mapHeight, mapWidth, latitudeStartingPoint, longitudeStartingPoint, startingDirection);

        output(PresentationMessage.READY_MESSAGE);
        reader.next(); //input ignored
    }

    private static int takeIntegerInput(Scanner reader) {
        try {
            return reader.nextInt();
        } catch (InputMismatchException e) {
            output("(please, introduce a number)");
            reader.next();
            return takeIntegerInput(reader);
        }
    }

    private static void deployRover(int mapHeight, int mapWidth, int latitudeStartingPoint, int longitudeStartingPoint, Direction startingDirection) {
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);
        var marsMap = new MarsMap(mapHeight, mapWidth, startingCoordinates);
        rover = new MarsRover(marsMap, startingDirection);
    }

    private static void commandLoop(Scanner reader) {
        var nextCommand = "none";
        do {
            output("Insert command (f = forward, b = backward, l = turn left, r = turn right, q = quit):");
            nextCommand = reader.next();
            switch (nextCommand) {
                case "f" -> rover.moveForward();
                case "b" -> rover.moveBackwards();
                case "l" -> rover.turnLeft();
                case "r" -> rover.turnRight();
                case "q" -> output("Disconnecting from Mars...");
                default -> output("unknown command");
            }
        } while (isNotExitSignal(nextCommand));
    }

}
